@@ -24,9 +24,12 @@
 This type must be recoverable from a :class:`Message` object, as petitions
 cannot be sent from clients to servers and vice versa.
 """
+from __future__ import annotations
+
 import subprocess
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
+from enum import IntEnum
 from functools import total_ordering
 from queue import Queue
 from typing import Any, Callable, NoReturn, Optional, Type, TypeVar, Union
@@ -54,6 +57,28 @@ inherits from it.
 """
 
 
+class PetitionState(IntEnum):
+    """
+    Class that contains the logic for determining the status of a
+    petition, allowing to determine if the state is valid. For example, a
+    running petition shouldn't be run again; a cancelled petition shouldn't
+    be started, etc.
+
+    .. versionadded:: 0.2.2
+    """
+
+    PENDING = 0
+    """The petition has just been created"""
+    ENQUEUED = 1
+    """The petition is already enqueued waiting for running"""
+    RUNNING = 2
+    """The petition has been started and it's running"""
+    FINISHED = 3
+    """Petition processing has been done"""
+    CANCELLED = 4
+    """Ff the petition has been cancelled before finishing processing"""
+
+
 @total_ordering
 @dataclass
 class Petition(ABC):
@@ -71,6 +96,8 @@ class Petition(ABC):
      + :attr:`action` represents the callable that will be executed.
      + :attr:`condition` is a predicate which defines whether the petition
        can be run or not.
+     + :attr:`state` is the internal state of the petition, defined by the
+       enum :class:`PetitionState`.
 
     This class is intended to be a stub so your implementation must inherit
     from this one.
@@ -81,16 +108,22 @@ class Petition(ABC):
         the ID (for equality/inequality tests) and the priority (for comparison
         tests).
 
-    .. versionchanged:: 0.2.0
+    .. versionadded:: 0.2.0
         This class now defines a :func:`terminate` which is abstract and should
         be overriden by all subclasses that inherit from :class:`Petition`. Such
         function allows defining a custom behavior when a finish message is
         received.
 
-        Warning:
+        .. warning::
             This is a breaking change - all existing projects should migrate
             their own :class:`Petition` subclasses to the new definition.
 
+    .. versionadded:: 0.2.2
+        Petition now have an internal state that is managed by the processor
+        that owns it (so the manager also). This internal state is intended to be
+        used for deciding if a petition should be run or not. By default, its
+        state is :obj:`PENDING <PetitionState.PENDING>`.
+
     :see: :py:func:`field <dataclasses.field>`
     """
 
@@ -155,6 +188,12 @@ class Petition(ABC):
 
     """
 
+    state: PetitionState = field(compare=False, init=False, default=PetitionState.PENDING)
+    """
+    Petition's state that indicates current step in the processing queue. Available
+    states are defined at :class:`PetitionState` enumerate.
+    """
+
     def communicate(self, message: Any, blocking: bool = True):
         """
         Communicates with the source process by sending a message through the internal queue.
@@ -215,8 +254,24 @@ class Petition(ABC):
             :obj:`bool`: that indicates if the operation has been successful or not.
 
         .. versionadded:: 0.2.0
+
+        .. versionchanged:: 0.2.2
+            This function now has a body which must be called always from child classes.
+            Such body ensures petition state is valid and sets the new one, which will
+            be :obj:`CANCELLED <PetitionState.CANCELLED>`.
         """
-        ...
+        if self.state in {PetitionState.FINISHED, PetitionState.PENDING}:
+            raise ValueError(
+                "Cannot terminate a petition whose state is either FINISHED or PENDING"
+            )
+
+        # if we are called, our state is now CANCELLED
+        if self.state in {PetitionState.RUNNING, PetitionState.ENQUEUED}:
+            self.state = PetitionState.CANCELLED
+        else:
+            raise AttributeError(f"Unknown petition state: {self.state}")
+
+        return True
 
     def __eq__(self, __o: object) -> bool:
         # ensure we are comparing against our class or a subclass of ours
@@ -289,8 +344,8 @@ class WatchdogPetition(Petition):
     action = None
     condition = None
 
-    def __init__(self):
-        pass
+    def __init__(self, queue: Optional[Queue] = None):
+        self.queue = queue
 
     def terminate(self, _: Optional[int]) -> bool:
         pass
@@ -327,7 +382,21 @@ class SignalingPetition(Petition):
     """
 
     def terminate(self, pid: Optional[int]) -> bool:
-        if pid is None:
+        """Sends the specified signal to the process and/or its parent.
+
+        Args:
+            pid (:obj:`int`): process identifier to send the signal to.
+
+        Raises:
+            ValueError: if the ``pid`` is :obj:`None`.
+
+        Returns:
+            :obj:`bool`: :obj:`True` if all children have been signaled correctly,
+                :obj:`False` otherwise.
+        """
+        valid = super().terminate(pid)
+
+        if pid is None and not valid:
             raise ValueError(f'Petition of type "{type(self).__name__}" requires a valid PID')
 
         return kill_proc_tree(pid, self.kill_parent, self.signal)
@@ -341,4 +410,5 @@ __all__ = [
     "ProcT",
     "WatchdogPetition",
     "SignalingPetition",
+    "PetitionState",
 ]
