@@ -23,21 +23,31 @@
 The class :class:`Processor` is responsible for handing queues, objects and petitions.
 Alongside with :class:`Manager <orcha.lib.Manager>`, it's the heart of the orchestrator.
 """
+from __future__ import annotations
+
 import multiprocessing
 import random
 import subprocess
+import typing
 from queue import PriorityQueue, Queue
 from threading import Event, Lock, Thread
-from time import sleep
 from typing import Dict, List, Optional, Tuple, Union
 
 import systemd.daemon as systemd
 
 from orcha import properties
 from orcha.interfaces.message import Message
-from orcha.interfaces.petition import EmptyPetition, Petition, WatchdogPetition
+from orcha.interfaces.petition import (
+    EmptyPetition,
+    Petition,
+    PetitionState,
+    WatchdogPetition,
+)
 from orcha.utils.logging_utils import get_logger
 
+if typing.TYPE_CHECKING:
+    from orcha.lib import Manager
+
 log = get_logger()
 
 
@@ -183,16 +193,17 @@ class Processor:
                                                    Defaults to :obj:`None`.
         manager (:class:`Manager`, optional): manager object used for synchronization and action
                                               calling. Defaults to :obj:`None`.
+        notify_watchdog (:obj:`bool`, optional): if the service is running under systemd,
+            notify periodically (every 5 seconds) that we are alive and doing things. If there
+            is any kind of unexpected error, a watchdog trigger will be set and the service
+            will be restarted.
+
         look_ahead (:obj:`int`, optional): amount of items to look ahead when querying the queue.
             Having a value higher than 1 allows the processor to access items further in the queue
             if, for any reason, the next one is not available yet to be executed but the second
             one is (i.e.: if you define priorities based on time, allow the second item to be
             executed before the first one). Take special care with this parameter as this may
             cause starvation in processes.
-        notify_watchdog (:obj:`bool`, optional): if the service is running under systemd,
-            notify periodically (every 5 seconds) that we are alive and doing things. If there
-            is any kind of unexpected error, a watchdog trigger will be set and the service
-            will be restarted.
 
     Raises:
         ValueError: when no arguments are given and the processor has not been initialized yet.
@@ -211,7 +222,7 @@ class Processor:
         self,
         queue: multiprocessing.Queue = None,
         finishq: multiprocessing.Queue = None,
-        manager=None,
+        manager: Optional[Manager] = None,
         look_ahead: int = 1,
         notify_watchdog=False,
     ):
@@ -219,18 +230,20 @@ class Processor:
             if not all((queue, finishq, manager)):
                 raise ValueError("queue & manager objects cannot be empty during init")
 
-            self.lock = Lock()
-            self.queue = queue
-            self.finishq = finishq
+            self._lock = Lock()
+            self._queue = queue
+            self._finishq = finishq
             self.manager = manager
             self.look_ahead = look_ahead
+            self._finished = Event()
             self.running = True
             self.notify_watchdog = notify_watchdog
+            """Whether to notify SystemD Watchdog periodically to inform about our status"""
 
             self._internalq = PriorityQueue()
             self._signals = Queue()
             self._threads: List[Thread] = []
-            self._petitions: Dict[int, Tuple[int, Petition]] = {}
+            self._petitions: Dict[Optional[int], Tuple[int, Petition]] = {}
             self._gc_event = Event()
             self._pred_lock = Lock()
             self._process_t = Thread(target=self._process)
@@ -238,33 +251,60 @@ class Processor:
             self._finished_t = Thread(target=self._signal_handler)
             self._signal_t = Thread(target=self._internal_signal_handler)
             self._gc_t = Thread(target=self._gc)
-            self._wd_t = Thread(target=self._notify_watchdog)
             self._process_t.start()
             self._internal_t.start()
             self._finished_t.start()
             self._signal_t.start()
             self._gc_t.start()
-            if self.notify_watchdog:
-                self._wd_t.start()
             self.__must_init__ = False
 
     @property
     def running(self) -> bool:
         """Whether if the current processor is running or not"""
-        return self._running
+        return not self._finished.is_set()
 
     @running.setter
     def running(self, v: bool):
-        with self.lock:
-            self._running = v
+        if v:
+            self._finished.clear()
+        else:
+            self._finished.set()
 
-    def exists(self, m: Union[Message, int, str]) -> bool:
+    @property
+    def look_ahead(self) -> int:
+        """Amount of items to look ahead when querying the queue"""
+        return self._look_ahead
+
+    @look_ahead.setter
+    def look_ahead(self, v: int):
+        with self._lock:
+            self._look_ahead = v
+
+    def wait(self, n: Optional[float] = None) -> bool:
+        """Waits the specified amount of time (in seconds) or until
+        the main process is done (it is, until :attr:`running` returns
+        :obj:`False`), the default.
+
+        .. versionadded:: 0.2.3
+
+        Args:
+            n (:obj:`float`, optional): time to wait, in seconds. Defaults to :obj:`None`.
+
+        Returns:
+            :obj:`bool`: the :attr:`running` value after the given time has passed.
+        """
+        return self._finished.wait(n)
+
+    def is_running(self, m: Union[Message, int, str]) -> bool:
         """
         Checks if the given message is running or not.
 
         .. versionchanged:: 0.1.6
            Attribute :attr:`m` now supports a :obj:`str` as ID.
 
+        .. versionadded:: 0.2.2
+            Renamed from :func:`exists` to :func:`is_running`.
+
         Args:
             m (:obj:`Message` | :obj:`int` | :obj:`str`]): the message to check or its
                 :attr:`id <orcha.interfaces.Message.id>` (if :obj:`int` or :obj:`str`).
@@ -286,7 +326,7 @@ class Processor:
         Args:
             m (Message): the message to enqueue
         """
-        self.queue.put(m)
+        self._queue.put(m)
 
     def finish(self, m: Union[Message, int, str]):
         """Sets a finish signal for the given message.
@@ -302,7 +342,7 @@ class Processor:
             m = m.id
 
         log.debug("received petition for finish message with ID %s", m)
-        self.finishq.put(m)
+        self._finishq.put(m)
 
     def _process(self):
         log.debug("fixing internal digest key")
@@ -311,15 +351,15 @@ class Processor:
         try:
             while self.running:
                 log.debug("waiting for message...")
-                m = self.queue.get()
+                m = self._queue.get()
                 if m is not None:
                     log.debug('converting message "%s" into a petition', m)
                     p: Optional[Petition] = self.manager.convert_to_petition(m)
                     if p is not None:
                         log.debug("> %s", p)
-                        if self.exists(p.id):
-                            log.warning("received message (%s) already exists", p)
-                            p.queue.put(f'message with ID "{p.id}" already exists\n')
+                        if self.is_running(p.id):
+                            log.warning("received message (%s) already running", p)
+                            p.queue.put(f'message with ID "{p.id}" is already running\n')
                             p.queue.put(1)
                             continue
                     else:
@@ -327,6 +367,8 @@ class Processor:
                         continue
                 else:
                     p = EmptyPetition()
+                p.state = PetitionState.ENQUEUED
+                self._petitions[p.id] = None, p
                 self._internalq.put(p)
         except Exception as e:
             log.fatal("unhandled exception: %s", e)
@@ -354,6 +396,9 @@ class Processor:
                     elif self.notify_watchdog and isinstance(p, WatchdogPetition):
                         log.debug("received watchdog request [WD is enabled for this instance]")
                         systemd.notify("WATCHDOG=1")
+                        # if there is a queue, we can finish the petition
+                        if p.queue is not None:
+                            p.finish(0)
 
                     if i > self._internalq.qsize():
                         break
@@ -365,7 +410,7 @@ class Processor:
                     self._threads.append(launch_t)
 
                 if not empty:
-                    sleep(random.uniform(0.5, 5))
+                    self.wait(random.uniform(0.5, 5))
             log.debug("internal process handler finished")
 
         except Exception as e:
@@ -415,7 +460,7 @@ class Processor:
         try:
             while self.running:
                 log.debug("waiting for finish message...")
-                m = self.finishq.get()
+                m = self._finishq.get()
                 self._signals.put(m)
         except Exception as e:
             log.fatal("unhandled exception: %s", e)
@@ -457,9 +502,6 @@ class Processor:
             p.communicate(f"Failed to finish petition with ID {p.id}\n")
             p.communicate(f"{e}\n")
         finally:
-            with self._pred_lock:
-                self.manager.on_finish(p)
-
             p.finish()
             self._petitions.pop(id, None)
             self._gc_event.set()
@@ -480,11 +522,6 @@ class Processor:
                 systemd.notify(f"STATUS=Failure due to unexpected exception - {e}")
                 systemd.notify("WATCHDOG=trigger")
 
-    def _notify_watchdog(self):
-        while self.running and self.notify_watchdog:
-            self._internalq.put(WatchdogPetition())
-            sleep(5)
-
     def shutdown(self):
         """
         Finishes all the internal queues and threads, waiting for any pending requests to
@@ -495,8 +532,8 @@ class Processor:
         try:
             log.info("finishing processor")
             self.running = False
-            self.queue.put(None)
-            self.finishq.put(None)
+            self._queue.put(None)
+            self._finishq.put(None)
             self._gc_event.set()
 
             log.info("waiting for pending processes...")
